import _ from 'lodash';
import { BootMixin } from '@loopback/boot';
import { BindingScope } from '@loopback/core';
import { SocketIoApplication } from '@loopback/socketio';
import debugFactory from 'debug';
import { SocketIoController } from './controllers';
import { RedisPubSubBindings, RedisPubSubComponent, RedisPubSubOptionsKeyBindings } from 'loopback4-redis-pubsub';
import { RedisPubSubLifecycleObserver, SocketIoLifecycleObserver } from './observers';
import { REDIS_PUBSUB_DATA_HANDLER, RedisPubSubDataHandlerProvider } from './services';
import { EntityEvent } from '@xtnd-dynamics/xd-data';
import path from 'path';
_.set(global, '__dirname', path.resolve());
const debug = debugFactory('loopback:example:socketio:demo');
export class ThirdCircleIoApplication extends BootMixin(SocketIoApplication) {
    constructor(options = {}) {
        super(options);
        this.projectRoot = __dirname;
        this.configure(RedisPubSubBindings.COMPONENT).to(options.redisPubSubOptions);
        this.component(RedisPubSubComponent);
        this.lifeCycleObserver(SocketIoLifecycleObserver);
        this.lifeCycleObserver(RedisPubSubLifecycleObserver);
        this.bind(REDIS_PUBSUB_DATA_HANDLER)
            .toProvider(RedisPubSubDataHandlerProvider)
            .tag({
            [RedisPubSubOptionsKeyBindings.NAMESPACE]: RedisPubSubBindings.REDIS_PUBSUB_NAMESPACE,
            [RedisPubSubOptionsKeyBindings.OPTIONS]: { channel: EntityEvent.DataChange },
        })
            .inScope(BindingScope.SINGLETON);
        this.socketServer.use((socket, next) => {
            debug('Global middleware - socket:', socket.id);
            next();
        });
        const ns = this.socketServer.route(SocketIoController);
        ns.use((socket, next) => {
            debug('Middleware for namespace %s - socket: %s', socket.nsp.name, socket.id);
            next();
        });
    }
}
//# sourceMappingURL=application.js.map