import { __decorate, __metadata, __param } from "tslib";
import _ from 'lodash';
import { service } from '@loopback/core';
import { socketio } from '@loopback/socketio';
import { RedisClient } from 'redis';
import { RedisPublisherClientService } from 'loopback4-redis-pubsub';
import { ThirdCircle } from '../types';
var Actions = ThirdCircle.IO.Actions;
var Channels = ThirdCircle.IO.RedisPubSub.Channels;
var Namespaces = ThirdCircle.IO.Namespaces;
var Zones = ThirdCircle.IO.Zones;
let SocketIoController = class SocketIoController {
    constructor(socket, redisPublisherClient) {
        this.socket = socket;
        this.redisPublisherClient = redisPublisherClient;
    }
    connect(socket) {
        console.debug('Client connected: %s', this.socket.id);
        socket.join(Zones.Default);
        const message = {
            action: Actions.Connect,
            socket: this.socket.id,
        };
        const messageStr = JSON.stringify(message);
        this.redisPublisherClient.publish(Channels.Application, messageStr);
        socket.emit('echo', this.socket.id);
    }
    // @socketio.emit('namespace' | 'requestor' | 'broadcast')
    registerChannel(msg) {
        console.debug('Subscribe to channel: %s', msg);
        if (Array.isArray(msg) && msg.length > 0) {
            msg.forEach(item => {
                this.socket.join(item);
            });
        }
        else {
            throw new Error('Channels data not appropriate');
        }
    }
    // @socketio.emit('namespace' | 'requestor' | 'broadcast')
    handleChatMessage(msg) {
        console.debug('General forwarded message: %s', msg);
        this.socket.nsp.emit('general-message-forward', msg);
    }
    // @socketio.emit('namespace' | 'requestor' | 'broadcast')
    handleGeneralMessage(msg) {
        console.debug('General Message : %s', msg);
        const parsedMsg = JSON.parse(msg);
        if (parsedMsg?.receiver?.to?.length > 0) {
            parsedMsg.receiver.to.forEach(item => this.socket.nsp.to(item.id).emit('message', {
                subject: parsedMsg.subject,
                body: parsedMsg.body,
                options: parsedMsg.options,
            }));
        }
        else {
            throw new Error('Inappropriate message data');
        }
    }
    logMessage(...args) {
        console.debug('Message: %s', args);
        const actions = args[0];
        const action = actions[0].toLowerCase();
        const message = { action: action, data: args };
        const messageStr = JSON.stringify(message);
        this.redisPublisherClient.publish(action, messageStr);
    }
    disconnect() {
        console.debug('Client disconnected: %s', this.socket.id);
        const message = {
            action: Actions.Disconnect,
            socket: this.socket.id,
        };
        const messageStr = JSON.stringify(message);
        this.redisPublisherClient.publish(Channels.Application, messageStr);
    }
    dataMessage(messages) {
        console.debug('Data: %s', messages.length);
        _.each(messages, message => {
            const messageStr = JSON.stringify(message);
            this.redisPublisherClient.publish('app', messageStr);
            this.redisPublisherClient.publish('dataChange', messageStr);
        });
    }
};
__decorate([
    socketio.connect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], SocketIoController.prototype, "connect", null);
__decorate([
    socketio.subscribe('subscribe-to-channel'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", void 0)
], SocketIoController.prototype, "registerChannel", null);
__decorate([
    socketio.subscribe('general-message-forward'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], SocketIoController.prototype, "handleChatMessage", null);
__decorate([
    socketio.subscribe('general-message'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], SocketIoController.prototype, "handleGeneralMessage", null);
__decorate([
    socketio.subscribe(/.+/),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], SocketIoController.prototype, "logMessage", null);
__decorate([
    socketio.disconnect(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SocketIoController.prototype, "disconnect", null);
__decorate([
    socketio.subscribe(/^data/),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", void 0)
], SocketIoController.prototype, "dataMessage", null);
SocketIoController = __decorate([
    socketio(Namespaces.Default),
    __param(0, socketio.socket()),
    __param(1, service(RedisPublisherClientService)),
    __metadata("design:paramtypes", [Object, RedisClient])
], SocketIoController);
export { SocketIoController };
//# sourceMappingURL=socketio.controller.js.map